//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

#include rmPaintBox.h 

class rmPaintbox (oP) of rmThemeAwarePaintBox(oP) custom

   this.Version = "1.06.1113"
   
   /*


	Classname   :  rmPaintbox.cc


   Date        :  May 2005


   For         :  dBASE Plus / dB2K / Win9x / NT / Win2000 / WinXP


   Author      :  Ronnie MacGregor

   Contact     :  rmPaintboxFeedback@dBASEdeveloper.co.uk

   Copyright   :  Ronnie MacGregor   All rights reserved.


   Disclaimer  :  It is the duty of the user of this code to determine its 
                  suitability for use.

                  The use of this code signifies agreement to the following
                  condition of use.
                  
                  In no circumstance will the author accept any responsibility
                  for any consequence arising from the use of this code.


   Description :  A generic PaintBox class which provides the developer with
                  many of the API functions that are required to do anything
                  useful with the dBASE PaintBox control.

                  It is intended that this class will be expanded as the need
                  arises. If you add an API extern or a function, please send
                  me a copy along with a small demo form showing how your
                  addition is used, and I'll be happy to add it to this base
                  class.

                  To check that you have the latest version of this control
                  go to:

                     www.dBASEdeveloper.co.uk 


   Useage      :  Use as a base class for paintbox controls

                     #include rmPaintBox.h

                     set procedure to rmPaintBox.cc additive

                     this.MyControl = new rmPaintBox(this)
                     with (this.MyControl)
                        onPaint = class::MyOnPaintCode
                        left = 0
                        top = 20
                        width = 200
                        height = 80
                     endwith


   Dependencies:

      rmPaintbox.h   - Header file


   Acknowledgements:

      Struct classes - Gary White

                       Modifications and additions by
                       Ronnie MacGregor and Marc Van den Berghen 

      LoadDLL()      - Marc Van den Berghen    


   Revisions:

      2005/06/18 

         Ronnie MacGregor
         PaintTransparentBitmap function added

      2005/11/16

         Ronnie MacGregor
         PaintTransparentSubBitmap function added

      2006/02/02

         Marc Van den Berghen
         Bug fix - hBMP was not being released in PaintTransparentBitmap
         and PaintTransparentSubBitmap

      2006/09/13

         This class now subclassed from rmThemeAwarePaintBox which has XP theme
         related custom properties that can be queried at paint time to
         establish which theme colour is in force and therfore allow conditional
         painting to match that theme colour. This means that any subclass of 
         this control taking advantage of this capability can respond to a change
         of theme colour mid session. See the rmThemeAwarePaintBox class header
         for more information.


   */

   // -------------------------------------

   // Stock properties
   this.height := 10
   this.width := 10
   // this.transparent := true
   // this.colorNormal := "" // Ensure transparency

   // -------------------------------------

   // API externs

   if type('AngleArc') # 'FP'
      extern clogical AngleArc(chandle, cint, cint, culong, cfloat, cfloat) gdi32
   endif

   if	type('Arc') # 'FP'
      extern clogical Arc(chandle, cint, cint, cint, cint, cint, cint, cint, cint) gdi32
   endif

   if type('ArcTo') # 'FP'
      extern clogical ArcTo(chandle, cint, cint, cint, cint, cint, cint, cint, cint) gdi32
   endif

   if type('BeginPaint') # 'FP'
      extern chandle BeginPaint(chandle, cptr) user32
   endif

   if type('BeginPath') # 'FP'
      extern clogical BeginPath(chandle) gdi32
   endif

   if type("BitBlt") # "FP"
      extern CLOGICAL BitBlt(;
         CHANDLE,            ; // HDC - hdcDest - handle to destination dc
         CINT,               ; // INT - nXDest - x-coord of destination upper-left corner
         CINT,               ; // INT - nYDest - y-coord of destination upper-left corner
         CINT,               ; // INT - nWidth - width of destination rectangle
         CINT,               ; // INT - nHeight - height of destination rectangle
         CHANDLE,            ; // HDC - hdcSrc - handle to source dc
         CINT,               ; // INT - nXSrc - x-coordinate of source upper-left corner
         CINT,               ; // INT - nYSrc - y-coordinate of source upper-left corner
         CLONG               ; // DWORD - dwRop - raster operation code
         ) gdi32
   endif

   if type('CloseEnhMetaFile') # 'FP'
      extern chandle CloseEnhMetaFile(chandle) gdi32
   endif

   if type('CloseFigure') # 'FP'
      extern clogical CloseFigure(chandle) gdi32
   endif

   if type('CloseMetaFile') # 'FP'
      extern chandle CloseMetaFile(chandle) gdi32
   endif

   if type('CopyEnhMetaFile') # 'FP'
      extern chandle CopyEnhMetaFile(chandle, cptr) gdi32 ;
         from 'CopyEnhMetaFileW'
   endif

   if type('CreateBitmap') # 'FP'
      extern chandle CreateBitmap(;
         CINT,                    ; // INT - nWidth - bitmap width in pixels
         CINT,                    ; // INT - nHeight - bitmap height in pixels
         CUINT,                   ; // UINT - cPlanes - number of colour planes
         CUINT,                   ; // UINT - cBitsPerPixel - number of bits to identify colour
         CPTR                     ; // CONST VOID - lpvBits - colour data array
         ) gdi32
   endif

   if type('CreateCompatibleBitmap') # 'FP'
      extern chandle CreateCompatibleBitmap(;
         CHANDLE,                           ; // HDC - hdc - handle to DC
         CINT,                              ; // INT - nWidth - bitmap width in pixels
         CINT                               ; // INT - nHeight - bitmap height in pixels
         ) gdi32                              
   endif

   if type("CreateCompatibleDC") # "FP"
      extern CHANDLE CreateCompatibleDC(CHANDLE) gdi32
   endif

   if type('CreateEnhMetaFile') # 'FP'
      extern chandle CreateEnhMetaFile(cptr, cptr, cptr, cptr) gdi32 ;
         from 'CreateEnhMetaFileW'
   endif

   if type('CreateFont') # 'FP'
      extern chandle CreateFont(cint, cint, cint, cint, cint, culong, ;
         culong, culong, culong, culong, culong, ;
         culong, culong, cstring) gdi32 ;
         from 'CreateFontA'
   endif

   if type('CreateMetaFile') # 'FP'
      extern chandle CreateMetaFile(cptr) gdi32 from 'CreateMetaFileW'
   endif

   if type('CreatePen') # 'FP'
      extern chandle CreatePen(cint, cint, culong) gdi32
   endif

   if type('CreatePolyRgn') # 'FP'
      extern chandle CreatePolygonRgn(cptr, cint, cint) gdi32
   endif

   if type('CreateRectRgn') # 'FP'
      extern chandle CreateRectRgn(cint, cint, cint, cint) gdi32
   endif

   if type('CreateSolidBrush') # 'FP'
      extern chandle CreateSolidBrush(culong) gdi32
   endif

   if type("DeleteDC") # "FP"
      extern CLOGICAL DeleteDC(CHANDLE) gdi32
   endif

   if type('DeleteEnhMetaFile') # 'FP'
      extern clogical DeleteEnhMetaFile(chandle) gdi32
   endif

   if type('DeleteMetaFile') # 'FP'
      extern clogical DeleteMetaFile(chandle) gdi32
   endif

   if type('DeleteObject') # 'FP'
      extern clogical DeleteObject(chandle) gdi32
   endif

   if type('DrawText') # 'FP'
      extern cint DrawText(chandle, cstring, cint, cptr, cint) user32 ;
         from 'DrawTextA'
   endif

   if type('Ellipse') # 'FP'
      extern clogical Ellipse(;
         chandle,             ; // HDC hdc         handle to DC
         cint,                ; // int nLeftRect   x-coord of upper-left corner of rectangle
         cint,                ; // int nTopRect    y-coord of upper-left corner of rectangle
         cint,                ; // int nRightRect  x-coord of lower-right corner of rectangle
         cint                 ; // int nBottomRect y-coord of lower-right corner of rectangle
         ) gdi32
   endif

   if type('EndPaint') # 'FP'
      extern cvoid EndPaint(chandle, cptr) user32
   endif 

   if type('EndPath') # 'FP'
      extern clogical EndPath(chandle) gdi32
   endif

   if type('ExtFloodFill') # 'FP'
      extern clogical ExtFloodFill(chandle, cint, cint, culong, cint) gdi32
   endif

   if type('FillRgn') # 'FP'
      extern clogical FillRgn(chandle, chandle, chandle) gdi32
   endif

   if type('FlattenPath') # 'FP'
      extern clogical FlattenPath(chandle) gdi32
   endif

   if type('FrameRgn') # 'FP'
      extern clogical FrameRgn(chandle, chandle, chandle, cint, cint) gdi32
   endif

   if type('GetClientRect') # 'FP'
      extern clogical GetClientRect(chandle, cptr) user32
   endif

   if type('GetCursorPos') # 'FP'
      extern clogical GetCursorPos(cptr) user32
   endif

   if type('GetDC') # 'FP'
      extern chandle GetDC(chandle) user32
   endif

   if type('GetDeviceCaps') # 'FP'
      extern cint GetDeviceCaps(chandle, cint) gdi32
   endif

   if type('GetEnhMetaFile') # 'FP'
      extern chandle GetEnhMetaFile(cstring) gdi32 ;
         from 'GetEnhMetaFileW'
   endif

   if type('GetMapMode') # 'FP'
      extern CINT GetMapMode(;
         chandle             ; // HDC - hdc - handle to device context
         ) gdi32
   endif

   if type("GetObject") # "FP"
      extern CINT GetObject( ;
         CHANDLE,            ; // handle to graphics object
         CINT,               ; // size of buffer for object information
         CPTR                ; // buffer for object information
         ) gdi32             ;
      from "GetObjectA" 
   endif

   if type('GetPixel') # 'FP'
      extern culong GetPixel(chandle, cint, cint) gdi32
   endif

   if type('GetTextColor') # 'FP'
      extern culong GetTextColor(chandle) gdi32
   endif

   if type('GetStockObject') # 'FP'
      extern chandle GetStockObject(cint) gdi32
   endif

   if type('GetSysColor') # 'FP'
      extern culong GetSysColor(cint) user32
   endif

   if type('GetTextExtentPoint32') # 'FP'
      extern clogical GetTextExtentPoint32(chandle, cstring, cint, cptr) gdi32 ;
         from 'GetTextExtentPoint32A'
   endif

   if type('GetVersion') # 'FP'
      extern culong GetVersion(cvoid) kernel32
   endif

   if type('GetWindowRect') # 'FP'
      extern clogical GetWindowRect(chandle, cptr) user32
   endif

   if type('InvalidateRect') # 'FP'
      extern clogical InvalidateRect(chandle, cptr, clogical) user32
   endif

   if type('LineTo') # 'FP'
      extern clogical LineTo(chandle, cint, cint) gdi32
   endif

   if type("LoadImage") # "FP"
      extern chandle LoadImage(;
         chandle,              ; // HINSTANCE hinst
         cstring,              ; // LPCTSTR lpszName
         cuint,                ; // UINT uType
         cint,                 ; // int cxDesired
         cint,                 ; // int cyDesired
         cuint                 ; // UINT fuLoad
         ) user32              ;
      from "LoadImageA"
   endif

   if type('LoadLibrary') # 'FP'
   	extern chandle LoadLibrary(cstring) kernel32 from 'LoadLibraryA'   
   endif

   if type('LoadFromResource') # 'FP'
 	   extern chandle LoadFromResource(chandle,cstring) user32 from "LoadBitmapA" 
   endif

   if type('LockWindowUpdate') # 'FP'
      extern clogical LockWindowUpdate(chandle) user32
   endif

   if type('MoveToEx') # 'FP'
      extern clogical MoveToEx(chandle, cint, cint, cptr) gdi32
   endif

   if type('MulDiv') # 'FP'
      extern cint MulDiv(cint, cint, cint) kernel32
   endif

   if type ('OffsetRgn') # 'FP'
      extern cint OffsetRgn(chandle, cint, cint) gdi32
   endif

   if type('PaintRgn') # 'FP'
      extern clogical PaintRgn(chandle, chandle) gdi32
   endif

   if type('PathToRegion') # 'FP'
      extern chandle PathToRegion(chandle) gdi32
   endif

   if type('Pie') # 'FP'
      extern clogical Pie(chandle, cint, cint, cint, ;
         cint, cint, cint, cint, cint) gdi32
   endif

   if type('PlayEnhMetaFile') # 'FP'
      extern clogical PlayEnhMetaFile(chandle, chandle, cptr) gdi32
   endif

   if type('PlayMetaFile') # 'FP'
      extern clogical PlayMetaFile(chandle, chandle) gdi32
   endif

   if type('PolyBezier') # 'FP'
      extern clogical PolyBezier(chandle, cptr, culong) gdi32
   endif

   if type('PolyBezierTo') # 'FP'
      extern clogical PolyBezierTo(chandle, cptr, culong) gdi32
   endif

   if type('Polygon') # 'FP'
      extern clogical Polygon(chandle, cptr, cint) gdi32
   endif

   if type('PtInRegion') # 'FP'
      extern clogical PtInRegion(chandle, cint, cint) gdi32
   endif

   if type('Rectangle') # 'FP'
      extern clogical Rectangle(chandle, cint, cint, cint, cint) gdi32
   endif

   if type('ReleaseDC') # 'FP'
      extern cint ReleaseDC(chandle, chandle) user32
   endif

   if type('ReleaseCapture') # 'FP'
      extern clogical ReleaseCapture(cvoid) user32
   endif

   if type("RoundRect") # "FP"
      extern clogical RoundRect(;
         chandle,               ; // hDC
         cint,                  ; // nTopLeftX
         cint,                  ; // nTopLeftY
         cint,                  ; // nBottRightX
         cint,                  ; // nBottRightY
         cint,                  ; // nRoundX
         cint                   ; // nRoundY
         ) gdi32
   endif

   if type('ScreenToClient') # 'FP'
      extern clogical ScreenToClient(chandle, cptr) user32
   endif

   if type('SelectObject') # 'FP'
      extern chandle SelectObject(chandle, chandle) gdi32
   endif

   if type('SetArcDirection') # 'FP'
      extern cint SetArcDirection(chandle, cint) gdi32
   endif

   if type('SetBkColor') # 'FP'
      extern culong SetBkColor(;
         chandle,              ; // HDC - hdc - handle to dc
         culong                ; // COLORREF - crColor - background colour value
         ) gdi32
   endif

   if type('SetBkMode') # 'FP'
      extern cint SetBkMode(chandle, cint) gdi32
   endif

   if type('SetCapture') # 'FP'
      extern chandle SetCapture(chandle) user32
   endif

   if type('SetMapMode') # 'FP'
      extern cint SetMapMode(;
         chandle,            ; // HDC - hdc - handle to device context 
         cint                ; // INT - fnMapMode - new mapping mode
         ) gdi32
   endif

   if type('SetPixel') # 'FP'
      extern culong SetPixel(chandle, cint, cint, culong) gdi32
   endif

   if type('SetTextAlign') # 'FP'
      extern cuint SetTextAlign(chandle, cuint) gdi32
   endif

   if type('SetTextColor') # 'FP'
      extern culong SetTextColor(chandle, culong) gdi32
   endif

   if type('ShellAbout') # 'FP'
      extern cint ShellAbout(chandle, cstring, cstring, cptr);
         shell32 from 'ShellAboutA'
   endif

   if type("StretchBlt") # "FP"
      extern CLOGICAL StretchBlt(CHANDLE, CINT, CINT, CINT, CINT, CHANDLE, CINT, CINT, CINT, CINT, CLONG) gdi32
   endif

   if type('StrokeAndFillPath') # 'FP'
      extern clogical StrokeAndFillPath(chandle) gdi32
   endif

   if type('TextOut') # 'FP'
      extern clogical TextOut(chandle, cint, cint, cptr, cint) gdi32 ;
         from 'TextOutW'
   endif

   // -------------------------------------

   function LoadDLL(cName, oRef)

      /*

      Author:      Marc Van den Berghen
      Date:		    May 2005

      Modified:    December 2005
                   by Ronnie MacGregor
                   to accept an object reference
      
      */

      // Reference the object
      if empty(argvector(2))
         oRef = this
      endif

      // Load dll to be used for bitmap resources
      try                                               
         oRef.hModule = LoadLibrary(cName)
      catch (exception e)	
         oRef.hModule = 0
      finally 
         if oRef.hModule == 0
            msgbox("Error during Resource Initialisation, file name - " + cName, "Warning", 0)
         endif
      endtry 

   return        

   // -------------------------------------

   function PaintBitmap(cBMPName, hDCDest, nXDest, nYDest, nDestWidth, nDestHeight, nXSource, nYSource)

      // Get named bitmap
      hBMP = LoadFromResource(this.hModule, cBMPName)

      // Get bitmap dimensions
      xBMP = new BitmapStruct()
      GetObject(hBMP, 24, xBMP)
      nBMPWidth = xBMP.getWidth()
      nBMPHeight = xBMP.getHeight()

      // Negative parameter means - "In from right or bottom edge"
      if nXSource < 0
         nXSource = nBMPWidth + nXSource
      endif
      if nYSource < 0
         nYSource = nBMPHeight + nYSource
      endif

      // Must select bitmap into a memory device context first
      hMemDC = CreateCompatibleDC(hDCDest)
      selectobject(hMemDC,hBMP)

      // Bit Block Transfer from memory device context to display device context
      BitBlt(hdcDest, nXDest, nYDest, nDestWidth, nDestHeight, hmemdc, nXSource, nYSource, SRCCOPY)

      // Clean up
      DeleteDC(hMemDC)
      DeleteObject(hBMP)  

   return

   // -------------------------------------

   function PaintSubBitmap(cBMPName, hDCDest, nXDest, nYDest, nSubBitmaps, nWhichSub)

      // Get named bitmap
      hBMP = LoadFromResource(this.hModule, cBMPName)

      // Get bitmap dimensions
      xBMP = new BitmapStruct()
      GetObject(hBMP, 24, xBMP)
      nBMPWidth = xBMP.getWidth()
      nBMPHeight = xBMP.getHeight()

      // Must select bitmap into a memory device context first
      hMemDC = CreateCompatibleDC(hDCDest)
      selectobject(hMemDC,hBMP)

      // Get Bitmap Section info
      if nSubBitmaps <> 0  
         nSectionHeight = nBMPHeight / nSubBitmaps
      else
         nSectionHeight = nBMPHeight
      endif

      if nWhichSub <> 0
         nSectionTop = ((nBMPHeight / nSubBitmaps) * nWhichSub)
      else
         nSectionTop = 0
      endif

      // Bit Block Transfer from memory device context to display device context
      BitBlt(hdcDest, nXDest, nYDest, nBMPWidth, nSectionHeight, hmemdc, 0, nSectionTop, SRCCOPY)

      // Clean up
      DeleteDC(hMemDC)
      DeleteObject(hBMP)  

   return

   // -------------------------------------

   function PaintSubBitmapStretch(cBMPName, hDCDest, nXDest, nYDest, nSubBitmaps, nWhichSub, nWidthDest, nHeightDest)

      // Get named bitmap
      hBMP = LoadFromResource(this.hmodule, cBMPName)

      // Get bitmap dimensions
      xBMP = new BitmapStruct()
      GetObject(hBMP, 24, xBMP)
      nBMPWidth = xBMP.getWidth()
      nBMPHeight = xBMP.getHeight()

      // Must select bitmap into a memory device context first
      hMemDC = CreateCompatibleDC(hDCDest)
      selectobject(hMemDC,hBMP)

      // Get Bitmap Section info
      if nSubBitmaps <> 0  
         nSectionHeight = nBMPHeight / nSubBitmaps
      else
         nSectionHeight = nBMPHeight
      endif

      if nWhichSub <> 0
         nSectionTop = ((nBMPHeight / nSubBitmaps) * nWhichSub)
      else
         nSectionTop = 0
      endif

      // Stretch Bit Block Transfer from memory device context to display device context
      StretchBlt(hDCDest, nXDest, nYDest, nWidthDest, nHeightDest, hMemDC, 0, nSectionTop, nBMPWidth, nSectionHeight, SRCCOPY)

      // Clean up
      DeleteDC(hMemDC)
      DeleteObject(hBMP)  

   return

   // -------------------------------------

   function PaintBitmapStretch(cBMPName, hDCDest, nXDest, nYDest, nWidthDest, nHeightDest, nXSource, nYSource, nWidthSource, nHeightSource)

      // Get named bitmap
      hBMP = LoadFromResource(this.hmodule, cBMPName)

      // Get bitmap dimensions
      xBMP = new BitmapStruct()
      GetObject(hBMP, 24, xBMP)
      nBMPWidth = xBMP.getWidth()
      nBMPHeight = xBMP.getHeight()

      // Negative parameter means - "In from right or bottom edge"
      if nXSource < 0
         nXSource = nBMPWidth + nXSource
      endif
      if nYSource < 0
         nYSource = nBMPHeight + nYSource
      endif

      // Must select bitmap into a memory device context first
      hMemDC = CreateCompatibleDC(hDCDest)
      selectobject(hMemDC,hBMP)

      // Stretch Bit Block Transfer from memory device context to display device context
      StretchBlt(hDCDest, nXDest, nYDest, nWidthDest, nHeightDest, hMemDC, nXSource, nYSource, nWidthSource, nHeightSource, SRCCOPY)

      // Clean up
      DeleteDC(hMemDC)
      DeleteObject(hBMP)  

   return

   // -------------------------------------

   function PaintTransparentBitmap(cBMPName, hDCDest, nXDest, nYDest, cHexTransparentColour)

      // Get named bitmap
      hBMP = LoadFromResource(this.hModule, cBMPName)

      // Get bitmap dimensions
      xBMP = new BitmapStruct()
      GetObject(hBMP, 24, xBMP)
      nBMPWidth = xBMP.getWidth()
      nBMPHeight = xBMP.getHeight()

      // Must select bitmap into a memory device context first
      hdcTemp = CreateCompatibleDC(hDCDest)

      // Select the bitmap
      SelectObject(hdcTemp, hBMP)    

      // Create some DCs to hold temporary data
      hdcBack = CreateCompatibleDC(hDCDest)
      hdcObject = CreateCompatibleDC(hDCDest)
      hdcMem = CreateCompatibleDC(hDCDest)
      hdcSave = CreateCompatibleDC(hDCDest)

      // Create a bitmap for each DC
      // Monochrome DC
      bmAndBack = CreateBitmap(nBMPWidth, nBMPHeight, 1, 1, NULL)
      bmAndObject = CreateBitmap(nBMPWidth, nBMPHeight, 1, 1, NULL)
      bmAndMem = CreateCompatibleBitmap(hDCDest, nBMPWidth, nBMPHeight)
      bmSave = CreateCompatibleBitmap(hDCDest, nBMPWidth, nBMPHeight)

      // Each DC must select a bitmap object to store pixel data.
      bmBackOld = SelectObject(hdcBack, bmAndBack)
      bmObjectOld = SelectObject(hdcObject, bmAndObject)
      bmMemOld = SelectObject(hdcMem, bmAndMem)
      bmSaveOld = SelectObject(hdcSave, bmSave)

      // Set proper mapping mode
      SetMapMode(hdcTemp, GetMapMode(hDCDest))

      // Save the bitmap sent here, because it will be overwritten
      BitBlt(hdcSave, 0, 0, nBMPWidth, nBMPHeight, hdcTemp, 0, 0, SRCCOPY)

      // Set the background Colour of the source DC to the colour
      // contained in the parts of the bitmap that should be transparent
      cColour = SetBkColor(hdcTemp, htoi(cHexTransparentColour))

      // Create the object mask for the bitmap by performing a BitBlt
      // from the source bitmap to a monochrome bitmap
      BitBlt(hdcObject, 0, 0, nBMPWidth, nBMPHeight, hdcTemp, 0, 0, SRCCOPY)

      // Set the background Colour of the source DC back to the original colour
      SetBkColor(hdcTemp, cColour)

      // Create the inverse of the object mask
      BitBlt(hdcBack, 0, 0, nBMPWidth, nBMPHeight, hdcObject, 0, 0, NOTSRCCOPY)

      // Copy the background of the main DC to the destination
      BitBlt(hdcMem, 0, 0, nBMPWidth, nBMPHeight, hDCDest, nXDest, nYDest, SRCCOPY)

      // Mask out the places where the bitmap will be placed
      BitBlt(hdcMem, 0, 0, nBMPWidth, nBMPHeight, hdcObject, 0, 0, SRCAND)

      // Mask out the transparent Coloured pixels on the bitmap
      BitBlt(hdcTemp, 0, 0, nBMPWidth, nBMPHeight, hdcBack, 0, 0, SRCAND)

      // XOR the bitmap with the background on the destination DC
      BitBlt(hdcMem, 0, 0, nBMPWidth, nBMPHeight, hdcTemp, 0, 0, SRCPAINT)

      // Copy the destination to the screen.
      BitBlt(hDCDest, nXDest, nYDest, nBMPWidth, nBMPHeight, hdcMem, 0, 0, SRCCOPY)

      // Place the original bitmap back into the bitmap sent here
      BitBlt(hdcTemp, 0, 0, nBMPWidth, nBMPHeight, hdcSave, 0, 0, SRCCOPY)

      // Delete the memory bitmaps
      DeleteObject(SelectObject(hdcBack, bmBackOld))
      DeleteObject(SelectObject(hdcObject, bmObjectOld))
      DeleteObject(SelectObject(hdcMem, bmMemOld))
      DeleteObject(SelectObject(hdcSave, bmSaveOld))

      // Delete the memory DCs
      DeleteDC(hdcMem)
      DeleteDC(hdcBack)
      DeleteDC(hdcObject)
      DeleteDC(hdcSave)
      DeleteDC(hdcTemp)

      // Delete hBMP
      DeleteObject(hBMP)

   return nBMPWidth

   // -------------------------------------

   function PaintTransparentSubBitmap(cBMPName, hDCDest, nXDest, nYDest, cHexTransparentColour, nSubBitmaps, nWhichSub)

      // Get named bitmap
      hBMP = LoadFromResource(this.hModule, cBMPName)

      // Get bitmap dimensions
      xBMP = new BitmapStruct()
      GetObject(hBMP, 24, xBMP)
      nBMPWidth = xBMP.getWidth()
      nBMPHeight = xBMP.getHeight()

      if nSubBitmaps <> 0  
         nSectionHeight = nBMPHeight / nSubBitmaps
      else
         nSectionHeight = nBMPHeight
      endif

      if nWhichSub <> 0
         nSectionTop = ((nBMPHeight / nSubBitmaps) * nWhichSub)
      else
         nSectionTop = 0
      endif

      // Must select bitmap into a memory device context first
      hdcTemp = CreateCompatibleDC(hDCDest)

      // Select the bitmap
      SelectObject(hdcTemp, hBMP)    

      // Create some DCs to hold temporary data
      hdcBack = CreateCompatibleDC(hDCDest)
      hdcObject = CreateCompatibleDC(hDCDest)
      hdcMem = CreateCompatibleDC(hDCDest)
      hdcSave = CreateCompatibleDC(hDCDest)

      // Create a bitmap for each DC
      // Monochrome DC
      bmAndBack = CreateBitmap(nBMPWidth, nSectionHeight, 1, 1, NULL)
      bmAndObject = CreateBitmap(nBMPWidth, nSectionHeight, 1, 1, NULL)
      bmAndMem = CreateCompatibleBitmap(hDCDest, nBMPWidth, nSectionHeight)
      bmSave = CreateCompatibleBitmap(hDCDest, nBMPWidth, nSectionHeight)

      // Each DC must select a bitmap object to store pixel data.
      bmBackOld = SelectObject(hdcBack, bmAndBack)
      bmObjectOld = SelectObject(hdcObject, bmAndObject)
      bmMemOld = SelectObject(hdcMem, bmAndMem)
      bmSaveOld = SelectObject(hdcSave, bmSave)

      // Set proper mapping mode
      SetMapMode(hdcTemp, GetMapMode(hDCDest))

      // Save the bitmap sent here, because it will be overwritten
      BitBlt(hdcSave, 0, 0, nBMPWidth, nSectionHeight, hdcTemp, 0, nSectionTop, SRCCOPY)

      // Set the background Colour of the source DC to the colour
      // contained in the parts of the bitmap that should be transparent
      cColour = SetBkColor(hdcTemp, htoi(cHexTransparentColour))

      // Create the object mask for the bitmap by performing a BitBlt
      // from the source bitmap to a monochrome bitmap
      BitBlt(hdcObject, 0, 0, nBMPWidth, nSectionHeight, hdcTemp, 0, nSectionTop, SRCCOPY)

      // Set the background Colour of the source DC back to the original colour
      SetBkColor(hdcTemp, cColour)

      // Create the inverse of the object mask
      BitBlt(hdcBack, 0, 0, nBMPWidth, nSectionHeight, hdcObject, 0, 0, NOTSRCCOPY)

      // Copy the background of the main DC to the destination
      BitBlt(hdcMem, 0, 0, nBMPWidth, nSectionHeight, hDCDest, nXDest, nYDest, SRCCOPY) 

      // Mask out the places where the bitmap will be placed
      BitBlt(hdcMem, 0, 0, nBMPWidth, nSectionHeight, hdcObject, 0, 0, SRCAND)

      // Mask out the transparent Coloured pixels on the bitmap
      BitBlt(hdcTemp, 0, nSectionTop, nBMPWidth, nSectionHeight, hdcBack, 0, 0, SRCAND) 

      // XOR the bitmap with the background on the destination DC
      BitBlt(hdcMem, 0, 0, nBMPWidth, nSectionHeight, hdcTemp, 0, nSectionTop, SRCPAINT) 

      // Copy the destination to the screen.
      BitBlt(hDCDest, nXDest, nYDest, nBMPWidth, nSectionHeight, hdcMem, 0, 0, SRCCOPY)

      // Place the original bitmap back into the bitmap sent here
      BitBlt(hdcTemp, 0, nSectionTop, nBMPWidth, nSectionHeight, hdcSave, 0, 0, SRCCOPY)

      // Delete the memory bitmaps
      DeleteObject(SelectObject(hdcBack, bmBackOld))
      DeleteObject(SelectObject(hdcObject, bmObjectOld))
      DeleteObject(SelectObject(hdcMem, bmMemOld))
      DeleteObject(SelectObject(hdcSave, bmSaveOld))

      // Delete the memory DCs
      DeleteDC(hdcMem)
      DeleteDC(hdcBack)
      DeleteDC(hdcObject)
      DeleteDC(hdcSave)
      DeleteDC(hdcTemp)

      // Delete hBMP
      DeleteObject(hBMP)

   return 

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class Point of struct

   /*

   Author:      Gary White
   Date:		    September 16, 2000

	Description: Basic structures for use in api calls.
      
   */

	this.string := space(4)

   function getX
   	return int(this.getLong(0))

   function setX(n)
   	return this.setLong(0, n)

   function getY
   	return int(this.getLong(4))

   function setY(n)
   	return this.setLong(4, n)

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class Rect of struct

   /*

   Author:      Gary White
   Date:		    September 16, 2000

	Description: Basic structures for use in api calls.
      
   */

	this.string := space(8)

   function getLeft
   	return int(this.getLong(0))

   function setLeft(n)
   	return this.setLong(0, n)

   function getTop
   	return int(this.getLong(4))

   function setTop(n)
   	return this.setLong(4, n)

   function getRight
   	return int(this.getLong(8))

   function setRight(n)
   	return this.setLong(8, n)

   function getBottom
   	return int(this.getLong(12))

   function setBottom(n)
   	return this.setLong(12, n)

   function getWidth
   	return int(this.getRight() - this.getLeft())

   function setWidth(n)
   	return this.setRight(this.getLeft() + n)

   function getHeight
   	return int(this.getBottom() - this.getTop())

   function setHeight(n)
   	return this.setBottom(this.getTop() + n)

   function deflate(l, t, r, b)
   	try
      	if argcount() = 1
         	t = l
            r = l
            b = l
         elseif argcount() = 2
         	r = l
            b = t
         endif
         this.setLeft(this.getLeft() + l)
         this.setTop(this.getTop() + t)
         this.setRight(max(this.getLeft(), this.getRight() - r))
         this.setBottom(max(this.getTop(), this.getBottom() - b))
      catch(exception e)
         ? e.message, e.lineno
      endtry
      return true

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class BitmapStruct of Struct

   /*

   Author:      Ronnie MacGregor

   Date:		    May 2005

	Description: Structure for retrieving Bitmap Information

   Uses Gary White's (modified by Ronnie) base Struct class 
      
   */

	this.string = space(24)

   function getType()
   	return int(this.getLong(0))

   function getWidth()
   	return int(this.getLong(4)) 

   function getHeight()
   	return int(this.getLong(8)) 

   function getWidthBytes()
   	return int(this.getLong(12)) 

   function getPlanes()
   	return int(this.getWord(16)) 

   function getBitsPixel()
   	return int(this.getWord(18)) 

   function getBits()
   	return this.getDWord(20)

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class Struct of string

   /*

   Author:      Gary White
   Date:		    September 16, 2000

	Description: Generic structure class used as superclass for other structs.

   Modified by Ronnie MacGregor and Marc Van den Berghen May 2005
      
   */

   // -------------------------------------

   function getWord(nIndex)
      // This function added by Ronnie MacGregor - May 2005 
   	local n
      n = this.getByte(nIndex) + ;
      	bitlshift(this.getByte(nIndex + 1), 8) 
   return n

   // -------------------------------------

   function getDWord(nIndex)
      // This function added by Ronnie MacGregor - May 2005
      // With information from Marc Van den Berghen   
   	local n
      n = this.getByte(nIndex) + ;
      	bitlshift(this.getByte(nIndex + 1), 8) + ;
      	bitlshift(this.getByte(nIndex + 2), 16) + ;
      	bitlshift(this.getByte(nIndex + 3), 24)
   return n

   // -------------------------------------

   function getLong(nIndex)
   	local n
      n = this.getByte(nIndex) + ;
      	bitlshift(this.getByte(nIndex + 1), 8) + ;
      	bitlshift(this.getByte(nIndex + 2), 16) + ;
      	bitlshift(this.getByte(nIndex + 3), 24)
      if bitset(n, 31)
         // This correction by Marc Van den Berghen May 2005 
         // n := - bitnot(n) // (Gary's original line)
         n := - bitnot(n) - 1 
      endif
   return n

   // -------------------------------------

   function setLong(nIndex, nValue)
   	local bRet
   	try
      	bRet = true
         this.setByte(nIndex, bitand(nValue, 0xFF))
         this.setByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))
         this.setByte(nIndex + 2, bitrshift(bitand(nValue, 0xFF0000), 16))
         this.setByte(nIndex + 3, bitrshift(bitand(nValue, 0xFF000000), 24))
      catch(exception e)
      	bRet = false
      endtry
   return bRet

   // -------------------------------------

   function release
      this.string = ''
      release object this
   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class rmThemeAwarePaintBox(oP) of PaintBox(oP) custom 

   /*
                                                                    
      VdB-Logic                                                     
																							 	
      MessageCatcher for WM_THEMECHANGED  			                  
                                                                     
      Programmer :  2005 - Marc Van den Berghen                    
                                                                    
      Version 1.06.0925                                                    
                                                                     
      The stock PaintBox is not aware of the message that is sent when   
      a system theme is changed. This custom PaintBox translates the
      WM_THEMECHANGED message into a middlemouseup message, that fires
      the dbase eventhandler.                            
                                                              		  
      Usage :  set procedure to rmThemeAwarePaintBox.cc additive              
                                                                    
               oRef = new rmThemeAwarePaintBox(this)                  	     
                                                                     
               Once you are sure that PaintBox has a handle then
               call the setup method.              
                                                                   
               oRef.StartThemeAwareness()


      2006 09 25 Ronnie MacGregor

         Restructured and expanded for use as a base class for the rmPaintBox.
         Marc's message catcher functionality now updates custom properties
         of this PaintBox class which can be evaluated in the onPaint method
         in order to provide the developer with the opportunity to paint
         according to the current XP theme colour, even if the theme colour
         is changed while the application is running.

   */

   // -------------------------------------

   // Custom Properties
   this.ThemeAware = false
   this.IsXP = false
   this.ThemeName = ""
   this.ThemeColorName = ""
   this.DefaultThemeColour = "METALLIC"


	// Custom Properties for Marc's message catcher
   this.wm			   	= 794  // WM_THEMECHANGED
	this.mad					= 0	 // address of 150 byte memory block	
   this.bytecode			= ''   // holds the assembler code
   this.wndproc			= 0    // address of the windows-procedure


   // Methods
   this.onClose = class::CleanupThemeAwareness
	   

   protect wm, mad, bytecode, wndproc, ptrtrafo


   // Custom methods
   this.StartThemeAwareness = class::StartThemeAwareness 
	this.CleanupThemeAwareness = class::CleanupThemeAwareness 


   // -------------------------------------

   function StartThemeAwareness()

      /*

      Programmer :  2005 - Marc Van den Berghen 

      The setup method must be called, when the paintbox has a handle.
      The assembler code is generated on the fly, depending on the 
      PaintBox handle, the address of the code in memory and the position of 
      the API-functions in memory.

      2006 09 25 Ronnie MacGregor

         Additions to integrate with this class as annotated.

      */

	 	#define GWL_WNDPROC   (-4)
	   #define GMEM_FIXED      0

      // Trap no valid handle
   	if this.hwnd == 0	
      	return false
      endif

      // Check for XP (Ronnie MacGregor) 
   	if this.WhatOS() == "XP"
         this.IsXP := true
         if type("GetCurrentThemeName") # "FP" 
            extern CUINT GetCurrentThemeName(CPTR,CUINT,CPTR,CUINT,CPTR,CUINT) uxtheme 
         endif
      else   	
      	return false
      endif

	   local tmp, cod, hpr, api,i

	   this.bytecode = replicate(chr(0),150)
	   cod= 'FFFFFFFFFFFFFFFFFFFFFFFF558BEC817D0C2B000000'
  		cod+='7515FF7514FF751068080200006834120000E821000000FF7514FF7510FF750C'
	   cod+='FF75086867450000E805000000C9C21000CCFF2500204000FF2504204000'

		extern chandle 	GlobalAlloc(CUINT, clong) kernel32
   	extern chandle    GlobalFree(chandle) kernel32
	   extern chandle    GetCurrentProcess( cvoid ) kernel32
   	extern clogical   WriteProcessMemory(chandle, cptr, cptr, clong,cptr ) kernel32
	   extern clong      GetWindowLong(chandle, CINT) user32 from "GetWindowLongA"
	   extern clong      SetWindowLong(chandle, CINT, CLONG) user32 from "SetWindowLongA" 
	   extern chandle    GetModuleHandle( cstring ) kernel32 from "GetModuleHandleA"
	   extern clong      GetProcAddress(chandle, cstring) kernel32
	   extern clong      GlobalLock(clong) kernel32

	   this.mad = globalalloc(GMEM_FIXED,300)
	   if this.mad == 0  // Should only happen, if there is no more free RAM
   	  	Msgbox("Memory allocation failed","Fatal Error",0)
      	return
	   endif

      // Get the address of the original window procedure of the PaintBox and 
      // change the assembler code to call it after it has done its work
   	this.wndproc=getwindowlong(this.hwnd,GWL_WNDPROC)
	   cod = stuff(cod,at('686745',cod)+2,8,class::ptrtrafo(this.wndproc))
	   cod = stuff(cod,at('683412',cod)+2,8,class::ptrtrafo(this.hwnd))
	   cod = stuff(cod,at('FF2500',cod)+4,8,class::ptrtrafo(this.mad))
	   cod = stuff(cod,at('FF2504',cod)+4,8,class::ptrtrafo(this.mad+4))
      cod = stuff(cod,at('7D0C2B',cod)+4,8,class::ptrtrafo(this.wm))
	   tmp = getmodulehandle('user32.dll')
   	cod = stuff(cod,1,8,class::ptrtrafo(getprocaddress(tmp,'CallWindowProcA')))
   	cod = stuff(cod,9,8,class::ptrtrafo(getprocaddress(tmp,'SendMessageA')))
   	for i = 0 to (len(cod)/2)-1 
    		this.bytecode.setbyte(i,htoi(substr(cod,(i*2)+1,2)))
	   endfor  
  		tmp = getcurrentprocess()
		writeprocessmemory(tmp,this.mad,this.bytecode,len(this.bytecode)*2,0)

      // Set the address of the new wndproc to point to the assembler code
	   setwindowlong(this.hwnd,GWL_WNDPROC,this.mad+12)

      // Set flag (Ronnie MacGregor)
      this.ThemeAware := true

      // Set event (Ronnie MacGregor)
      this.onMiddleMouseUp = class::UpdateThemeInfo

      // Update Theme Properties (Ronnie MacGregor)
      this.UpdateThemeInfo()

	return true

   // -------------------------------------

	function ptrtrafo(n)

      /*

      Programmer :  2005 - Marc Van den Berghen

      Helper method, to insert the hexcode in the assembler code.

      */

		local cret,c
		c=right('00000000'+itoh(n),8)
	   cret=right(c,2)+substr(c,5,2)+substr(c,3,2)+left(c,2)

	return cret

   // -------------------------------------

	function UpdateThemeInfo()

      /*

      Ronnie MacGregor

      Gets current theme info and updates custom class properties.

      */

      local cThemeFileName, cColorName, cSizeName  

      cThemeFileName = space(260)
      cColorName = space(260)
      cSizeName = space(260)

      this.ThemeName = ""
      this.ColorName = ""
      this.SizeName = ""

      nRet = GetCurrentThemeName(cThemeFileName, 260, cColorName, 260, cSizeName, 260)

      if nRet == 0

         if "LUNA" $ upper(trim(cThemeFileName))
            this.ThemeName := "LUNA"
         endif
         do case
            case "NormalColor" $ cColorName
               this.ThemeColorName := "BLUE"
            case "Metallic" $ cColorName
               this.ThemeColorName := "METALLIC"
            case "HomeStead" $ cColorName
               this.ThemeColorName := "HOMESTEAD"
         endcase

      endif

   return 

   // -------------------------------------

	function CleanupThemeAwareness()
   
      /*

      Programmer :  2005 - Marc Van den Berghen

      Use to free the memory used by the assembler program.
      Should be called when the PaintBox is no longer needed.

      2006 09 25 Ronnie MacGregor

         Changed to only run if required, and by default to fire
         in the onClose of the PaintBox.

      */


      if this.ThemeAware == true

         setwindowlong(this.hwnd,GWL_WNDPROC, this.wndproc)
         globalfree(this.mad)

      endif

   return

   // -------------------------------------

   function SetDefaultThemeColour(cColour)

      /*

      Set default XP colour property

      Ronnie MacGregor

      */

      do case
         case upper(cColour) == "METALLIC" or upper(cColour) == "SILVER"
            this.DefaultThemeColour := "METALLIC"
         case upper(cColour) == "HOMESTEAD" or upper(cColour) == "OLIVE GREEN"
            this.DefaultThemeColour := "HOMESTEAD"
         otherwise
            this.DefaultThemeColour := "BLUE"
      endcase

   return

   // -------------------------------------

   function WhatOS()

      /*

      Code by Rich at AutoTraker Inc.

      Jan 2006 updated by Ronnie MacGregor

         Windows 2003 detection added.
         Now returns false if fails to identify OS.

   
      Windows 95      = "Windows version 4.00"
      Windows 98      = "Windows version 4.10"
      Windows ME      = "Windows version 4.90"

      Windows NT3     = "Windows NT version 3.51"
      Windows NT4     = "Windows NT version 4.00"
      Windows 2000    = "Windows NT version 5.00"
      Windows XP      = "Windows NT version 5.01"
      Windows 2003    = "Windows NT version 5.02"
      Windows 2003 R2 = "Windows NT version 5.02"

      */

      local cOS

      cOS = false

      do case

         case "4.00"$ os() and not "NT"$ os()
            cOS = "95"

         case "4.10"$ os()
            cOS = "98"

         case "4.90"$ os()
            cOS = "ME"

         case ("4.00"$ os() or "3.51"$ os()) and "NT"$ os()
            cOS = "NT"

         case "5.00"$ os() and ("NT"$ os() or "2K"# os())
            cOS = "2000"

         case "5.01"$ os() and ("NT"$ os() or "XP"$ os())
            cOS = "XP"

         case "5.02"$ os() and ("NT"$ os() or "2K"$ os())
            cOS = "2003"

      endcase

   return cOS

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

// eof